[PATCH 3/3] EDAC: i5400: disable error reporting at teardown and refactor helper
From: Tushar Tibude
Date: Thu Apr 30 2026 - 04:51:09 EST
If error reporting is enabled during initialization but initialization
fails immediately after, or during normal driver teardown, error reporting
is left enabled in the mask register even after exit.
Replace i5400_enable_error_reporting() with i5400_set_error_reporting()
to combine enabling/disabling. Disable reporting at initialization
failure and driver exit, before call to i5400_put_devices() for cleanup.
This ensures clean hardware handling by disabling any unused error
reporting bits before exiting.
Signed-off-by: Tushar Tibude <tushar.tibude1000@xxxxxxxxx>
---
drivers/edac/i5400_edac.c | 33 ++++++++++++++++++++++++---------
1 file changed, 24 insertions(+), 9 deletions(-)
diff --git a/drivers/edac/i5400_edac.c b/drivers/edac/i5400_edac.c
index fb49a1d1d..ae4f92989 100644
--- a/drivers/edac/i5400_edac.c
+++ b/drivers/edac/i5400_edac.c
@@ -353,6 +353,9 @@ struct i5400_pvt {
/* Actual values for this controller */
int maxch; /* Max channels */
int maxdimmperch; /* Max DIMMs per channel */
+
+ /* Hardware error reporting status */
+ bool enabled_error_reporting;
};
/* I5400 MCH error information retrieved from Hardware */
@@ -1223,10 +1226,10 @@ static int i5400_init_dimms(struct mem_ctl_info *mci)
}
/*
* i5400_enable_error_reporting
* Turn on the memory reporting features of the hardware
*/
-static void i5400_enable_error_reporting(struct mem_ctl_info *mci)
+static void i5400_set_error_reporting(struct mem_ctl_info *mci, bool enable)
{
struct i5400_pvt *pvt;
u32 fbd_error_mask;
@@ -1237,8 +1240,11 @@ static void i5400_enable_error_reporting(struct mem_ctl_info *mci)
pci_read_config_dword(pvt->branchmap_werrors, EMASK_FBD,
&fbd_error_mask);
- /* Enable with a '0' */
- fbd_error_mask &= ~(ENABLE_EMASK_ALL);
+ /* Enable with 0, disable with 1 */
+ if (enable)
+ fbd_error_mask &= ~(ENABLE_EMASK_ALL);
+ else
+ fbd_error_mask |= ENABLE_EMASK_ALL;
pci_write_config_dword(pvt->branchmap_werrors, EMASK_FBD,
fbd_error_mask);
@@ -1319,17 +1325,19 @@ static int i5400_probe1(struct pci_dev *pdev, int dev_idx)
if (i5400_init_dimms(mci)) {
edac_dbg(0, "MC: Setting mci->edac_cap to EDAC_FLAG_NONE because i5400_init_dimms() returned nonzero value\n");
mci->edac_cap = EDAC_FLAG_NONE; /* no dimms found */
+ pvt->enabled_error_reporting = false;
} else {
edac_dbg(1, "MC: Enable error reporting now\n");
- i5400_enable_error_reporting(mci);
+ i5400_set_error_reporting(mci, true);
+ pvt->enabled_error_reporting = true;
}
/* add this new MC control structure to EDAC's list of MCs */
if (edac_mc_add_mc(mci)) {
edac_dbg(0, "MC: failed edac_mc_add_mc()\n");
- /* FIXME: perhaps some code should go here that disables error
- * reporting if we just enabled it
- */
+ /* Disable error reporting if we just enabled it */
+ if (pvt->enabled_error_reporting)
+ i5400_set_error_reporting(mci, false);
goto fail1;
}
@@ -1387,6 +1395,7 @@ static int i5400_init_one(struct pci_dev *pdev, const struct pci_device_id *id)
static void i5400_remove_one(struct pci_dev *pdev)
{
struct mem_ctl_info *mci;
+ struct i5400_pvt *pvt;
edac_dbg(0, "\n");
@@ -1397,6 +1406,12 @@ static void i5400_remove_one(struct pci_dev *pdev)
if (!mci)
return;
+ pvt = mci->pvt_info;
+
+ /* Disable error reporting on teardown */
+ if (pvt->enabled_error_reporting)
+ i5400_set_error_reporting(mci, false);
+
/* retrieve references to resources, and free those resources */
i5400_put_devices(mci);
@@ -1223,10 +1226,10 @@ static int i5400_init_dimms(struct mem_ctl_info *mci)
}
/*
- * i5400_enable_error_reporting
- * Turn on the memory reporting features of the hardware
+ * i5400_set_error_reporting
+ * Turn on/off the memory reporting features of the hardware
*/
--
2.43.0